"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = require("react");
var MapForState = (function () {
    function MapForState(state, setState) {
        this._state = state;
        this._setState = setState;
    }
    Object.defineProperty(MapForState.prototype, "size", {
        get: function () {
            return this.keys.length;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MapForState.prototype, "keys", {
        get: function () {
            return Object.keys(this._state);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MapForState.prototype, "values", {
        get: function () {
            return Object.values(this._state);
        },
        enumerable: true,
        configurable: true
    });
    MapForState.prototype.get = function (key) {
        return this._state[key];
    };
    MapForState.prototype.set = function (key, value) {
        this._setState(function (prevState) {
            var _a;
            return (__assign(__assign({}, prevState), (_a = {}, _a[key] = value, _a)));
        });
    };
    MapForState.prototype.has = function (key) {
        return this._state.hasOwnProperty(key);
    };
    MapForState.prototype.clear = function () {
        this._setState({});
    };
    MapForState.prototype.delete = function (key) {
        if (this.has(key)) {
            this._setState(function (prevState) {
                var otherKeys = Object.keys(prevState).filter(function (k) { return k !== key; });
                return otherKeys.reduce(function (newState, currentKey) {
                    newState[key] = prevState[currentKey];
                    return newState;
                }, {});
            });
            return true;
        }
        return false;
    };
    MapForState.prototype.entries = function () {
        var _this = this;
        var iteratorGenerator = function () {
            var index = 0;
            return {
                next: function () {
                    var key = _this.keys[index++];
                    var value = _this._state[key];
                    return [key, value];
                }
            };
        };
        return iteratorGenerator();
    };
    return MapForState;
}());
exports.MapForState = MapForState;
var useMap = function (initialValue) {
    if (initialValue === void 0) { initialValue = {}; }
    var _a = react_1.useState(initialValue), map = _a[0], setValues = _a[1];
    var keys = react_1.useMemo(function () { return Object.keys(map); }, [map]);
    var values = react_1.useMemo(function () { return Object.values(map); }, [map]);
    var size = react_1.useMemo(function () { return keys.length; }, [map]);
    var get = function (key) {
        return map[key];
    };
    var set = function (key, value) {
        setValues(function (prevState) {
            var _a;
            return (__assign(__assign({}, prevState), (_a = {}, _a[key] = value, _a)));
        });
    };
    var has = function (key) {
        return keys.includes(key);
    };
    var remove = function (key) {
        if (has(key)) {
            setValues(function (prevState) {
                var otherKeys = Object.keys(prevState).filter(function (k) { return k !== key; });
                return otherKeys.reduce(function (newState, currentKey) {
                    newState[key] = prevState[currentKey];
                    return newState;
                }, {});
            });
        }
    };
    return { keys: keys, values: values, size: size, get: get, setValues: setValues, set: set, has: has, remove: remove };
};
exports.default = useMap;
